	subroutine tension2 (x,xx,px,ileft,cx,ileftc,dim,spl)

!
!   Calculating the second order a) Chebyshev tension spline or
!                                b) tension spline curves
!
!   INPUT:
!      x      = double precision variable, the point at which the
!               spline curve is evaluated
!      xx     = a) rank-one double precision array of size dimx, the
!               increasing array of knots, assumed to fulfill the
!               conditions:
!               xx(1)=xx(2)<xx(3)<...<xx(i)<xx(i+1)<...<xx(dimx-1)=
!               xx(dimx); xx(3),...,xx(dimx-2) is the partition of
!               [xx(1),xx(dimx)]; the associated extended
!               partition T has all interior knots of multiplicity
!               2, and the boundary points of multiplicity 4;
!               b) rank-one double precision array of size dimx+4,
!               the increasing array of knots, assumed to fulfill
!               the conditions:
!               xx(1)=xx(2)=xx(3)=xx(4)<xx(5)<...<xx(i)<xx(i+1)<...
!               <xx(dimx)<xx(dimx+1)=xx(dimx+2)=xx(dimx+3)=
!               xx(dimx+4); xx(1),...,xx(dimx+4) is the extended
!               partition with all interior knots of multiplicity 1,
!               and the boundary points of multiplicity 4
!      px     = rank-one double precision array of size
!               a) dimx-1, b) dimx+3,
!               the array of tension parameters, assumed to be
!               nonnegative: px(i)>=0 is the tension parameter on
!               the subinterval [xx(i),xx(i+1));
!               a) px(1) and px(dimx-1) can be arbitrary
!               b) px(1),px(2),px(3),px(dimx+1),px(dimx+2) and
!               px(dimx+3) can be arbitrary
!      ileft  = integer variable, the index of the array xx such
!               that x is contained in the subinterval
!               [xx(ileft),xx(ileft+1)]
!      cx     = a) rank-two double precision array of shape dim by
!               2*dimx-4, the array of de Boor pints of the second
!               order Chebyshev tension spline associated with the
!               extended partition T;
!               cx(1:dim,i) corresponds to the B-spline with the
!               support [T(i),T(i+2)], so cx(1:dim,1) and
!               cx(2:dim,2) can be arbitrary
!               b) rank-two double precision array of shape dim by
!               dimx, the array of de Boor points of the second
!               order tension spline associated with the extended
!               partition xx;
!               cx(1:dim,i) corresponds to the B-spline with the
!               support [xx(i),xx(i+2)], so cx(1:dim,1) and
!               cx(2:dim,2) can be arbitrary
!      ileftc = a) integer variable, the index in the second
!               dimension of the array cx such that cx(1:dim,ileftc)
!               corresponds to the B-spline with the support
!               [T(ileftc),T(ileftc+2)] where
!               T(ileftc-1)=T(ileftc)=xx(ileft)
!               b) ileftc=ileft
!      dim    = integer variable, the extent of the first dimension
!               in the array cx, assumed to be positive
!
!   AUXILIARY TERM:
!      dimx   = a) size(xx)
!               b) size(xx)-4
!
!   OUTPUT:
!       spl   = rank-one double precision array of size dim, the
!               output value of the second order (Chebyshev)
!               tension spline curve at the point x
!

	use functions

	implicit none

	integer, intent(in) :: ileft,ileftc,dim
	double precision, intent(in) :: x,xx(*),px(*),cx(dim,*)	
	double precision, intent(out) :: spl(dim)

	double precision :: t(0:1),p,c(dim,-1:0) ! t(0)<=x<=t(1)
	double precision :: h1,h2,temp1,temp2

!
!	Storing only relevant data
!
	t(0:1)=xx(ileft:ileft+1)
	p=px(ileft)
	c(1:dim,-1:0)=cx(1:dim,ileftc-1:ileftc)
!
!	Calculating the length of subintervals involved
!
	h1=x-t(0)
	h2=t(1)-x
!
!	Direct calcualtion of 2 nontrivial B-splines at the point x
!
	temp1=dexp(-p*h1)*u(h2,h1,p)
	temp2=dexp(-p*h2)*u(h1,h2,p)
!
!	The final value of the spline curve
!
	spl=c(1:dim,-1)*temp1+c(1:dim,0)*temp2


	end subroutine tension2